import requests
import datetime
import json
import matplotlib.pyplot as plt
from collections import defaultdict

def get_recent_wikipedia_edits():
    url = "https://en.wikipedia.org/w/api.php"
    end_time = datetime.datetime.utcnow()
    start_time = end_time - datetime.timedelta(days=7)
    
    params = {
        "action": "query",
        "format": "json",
        "list": "recentchanges",
        "rcstart": end_time.strftime("%Y-%m-%dT%H:%M:%SZ"),
        "rcend": start_time.strftime("%Y-%m-%dT%H:%M:%SZ"),
        "rclimit": "200",
        "rcprop": "timestamp"
    }
    
    edit_counts = defaultdict(int)
    
    while True:
        try:
            response = requests.get(url, params=params, timeout=10)  # 10-second timeout
            if response.status_code != 200:
                print("Failed to fetch data from Wikipedia API")
                return {}
            
            data = response.json()
            
            if "query" in data:
                for edit in data["query"]["recentchanges"]:
                    timestamp = edit["timestamp"]
                    edit_date, edit_time = timestamp.split("T")
                    edit_hour, edit_minute = map(int, edit_time[:5].split(":"))
                    
                    # Corrected time filtering: Only count edits between 08:00 - 08:30 UTC
                    if edit_hour == 8 and 0 <= edit_minute < 30:
                        edit_counts[edit_date] += 1
            
            if "continue" in data:
                params["rccontinue"] = data["continue"].get("rccontinue", "")
            else:
                break
            
            print(f"Fetched {len(edit_counts)} edits so far...")  # Debugging progress
            
        except requests.exceptions.RequestException as e:
            print(f"Request failed: {e}")
            return {}
    
    return dict(edit_counts)

def visualize_data(edit_data):
    if not edit_data:
        print("No edit data available for visualization.")
        return
    
    dates = sorted(edit_data.keys())
    counts = [edit_data[date] for date in dates]
    
    plt.figure(figsize=(10, 5))
    plt.bar(dates, counts, color='blue')
    plt.xlabel("Date")
    plt.ylabel("Number of Edits")
    plt.title("Wikipedia Edits Between 8:00 - 8:30 AM (UTC) in the Last 7 Days")
    plt.xticks(rotation=45)
    plt.grid(axis='y')
    plt.show()

if __name__ == "__main__":
    print("Fetching Wikipedia edits between 8:00 - 8:30 AM (UTC) in the last 7 days...")
    edit_data = get_recent_wikipedia_edits()
    
    if edit_data:
        with open("wikipedia_edits.json", "w") as f:
            json.dump(edit_data, f, indent=4)
        
        for date, count in sorted(edit_data.items()):
            print(f"{date}: {count} edits")
        
        visualize_data(edit_data)
    else:
        print("No data retrieved. Check your internet connection or API status.")